import pygame.image
import os, math



class Sprite(object):
    x = 0
    y = 0
    Vx, Vy = 0,0
    connections = None
    _done_iter_stick = False

    def check_collision(self,other):
        if self.x < other.x + other.width:
            if self.y < other.y + other.height:
                if self.y > other.y - self.height:
                    if self.x > other.x - self.width:
                        return True

    def __init__(self, scr):
        self.screen = scr
        self.surface = pygame.image.load(os.path.join("sprites",self.img))
        #self.surface.set_colorkey((0,0,0))
        self.width, self.height = self.surface.get_size()
        self.connections = []


    def new_iter(self):
        """Called every itteration - stops itterative @stick method 
        from itterating infinately"""
        self._done_iter_stick = False

    def connect(self,other):
        #if self._done_iter_stick:
        #    return
        if not other in self.connections:
            self.connections.append(other)
        if not self in other.connections:
            other.connections.append(self)

        self._done_iter_stick = True
        if not other._done_iter_stick:
            other.connect(self)

        #momentum = abs(self.Vx) + abs(other.Vy)

        Vx = (self.Vx + other.Vx)/2
        Vy = (self.Vy + other.Vy)/2

        self.Vx = Vx
        self.Vy = Vy
        # move back to collision:
        r1 = max(self.width, self.height)/2
        r2 = max(other.width, other.height)/2
        desired_len = ((r1*r1) + (r2*r2) )

        diffx = (self.x - other.x)
        diffy = (self.y - other.y)
        desired_len = 0.95*math.sqrt(desired_len/(diffy*diffy + diffx*diffx))
        self.x = other.x + desired_len*diffx
        self.y = other.y + desired_len*diffy
        


    def stick(self):
        """Stick in place - e.g. to wall"""
        self.Vx=0
        self.Vy=0
        self._done_iter_stick = True
        for spr in [s for s in self.connections if not s._done_iter_stick]:
            spr.stick()

    def check_collision_radial(self,other, ignore_radius=0):
        """Radial collision detection"""
        r1 = max(self.width, self.height)/2
        r2 = max(other.width, other.height)/2

        check_len = (r1*r1) + (r2*r2) - 2* ignore_radius*ignore_radius
        if (self.x-other.x)**2 + (self.y-other.y)**2 <= check_len:
            return True
        return False


    def render(self, parent):
        try:
            if self.screen:
                if self.x < self.screen.width and self.y < self.screen.height and self.y > -self.width:
                    parent.blit(self.surface,(self.x,self.y) )
            else:
                parent.blit(self.surface,(self.x,self.y) )
        except Exception, e: print "Error:",self.x, self.y, e
        self.new_iter()

    def move(self):
        self.x+=self.Vx
        self.y+=self.Vy
        
        if self.x < 5:
            self.stick()

        if self.x >= (self.screen.width - self.width) or self.x < 0:
            self.bounce_x()
        if self.y >= (self.screen.height - self.height) or self.y < 0:
            self.bounce_y()

    def bounce_x(self):
        if self.x == 0:
            self.x = 1
        self.Vx = -(self.x/abs(self.x)) * abs(self.Vx)
        self._done_iter_stick = True
        for spr in [s for s in self.connections if not s._done_iter_stick]:
            spr.bounce_x()

    def bounce_y(self):
        if self.y == 0:
            self.y = 1
        self.Vy = -(self.y/abs(self.y)) *  abs(self.Vy)
        self._done_iter_stick = True
        for spr in [s for s in self.connections if not s._done_iter_stick]:
            spr.bounce_y()
            

class CellNeucleus(Sprite):
    img="cell_neucleus.png"
