#!/usr/bin/python

"""
Tim Wintle - for Mini-LD #6 09/01/2008 - 11/01/2008
"""

import pygame, time, random, math, os
import game.enemy, game.pill, game.player, game.scoreboard
from pygame.locals import *


class Screen(object):
    width = 800
    height = 600
    def __init__(self):
        screensize = (self.width,self.height)
        self.surface = pygame.display.set_mode(screensize, pygame.DOUBLEBUF)

class Sound(object):
    def __init__(self):
        pygame.mixer.init()
        self._gurgle1 = pygame.mixer.Sound(os.path.join("sounds","gurgle.ogg"))
        self._gurgle2 = pygame.mixer.Sound(os.path.join("sounds","gurgle2.ogg"))
        self._shot =    pygame.mixer.Sound(os.path.join("sounds","shot.ogg"))

    def gurgle(self):
        try:
            self._gurgle1.play()
        except:pass

    def gurglelong(self):
        try:
            self._gurgle2.play()
        except:pass

    def shot(self):
        try:
            self._shot.play()
        except:pass

    
    

def main():
    screen = Screen()
    
    surface = screen.surface

    pygame.mouse.set_visible(1)
    pygame.display.set_caption("Infection Fighter")
    main_game(screen)


def main_game(screen):

    surface = screen.surface

    surface.fill((165,255,165))

    clock = pygame.time.Clock()

    sound = Sound()

    enemies = []
    pills = []


    exit_game = False

    last_shot = 0

    counter = 0
    enemy_speed = 2
    difficulty_modulo = 60
    rotating = 0

    player = game.player.Player(screen)
    player.x = screen.width/2 - player.width/2
    player.y = screen.height/2 - player.height/2

    def random_enemy_class():
        return random.choice( [\
                game.enemy.SkullEnemy,
                game.enemy.CrossEnemy,
                game.enemy.ExclaimEnemy])

    score = game.scoreboard.Scoreboard(screen)

    player_dead = False

    while not exit_game:
        clock.tick(20)
        t = time.time()

        pygame.display.flip()

        if not player_dead:
            if (counter)%difficulty_modulo == 0:
                if counter%(2%difficulty_modulo) == 0 and difficulty_modulo > 12:
                    difficulty_modulo -=1
                # Spawn a new enemy
                angle = (random.random() * 0.75 * math.pi) + 0.75* math.pi #+ 1.5 *  math.pi
                e = random_enemy_class()(screen)
                angle2 = angle + 1.5*random.random() - 0.75
                e.Vx = math.cos(angle2) * enemy_speed
                e.Vy = math.sin(angle2) * enemy_speed
                rad = math.sqrt((screen.width/2)**2 + (screen.height/2)**2)
                x = screen.width/2 + rad* math.cos(angle + math.pi)
                y = screen.height/2 + rad* math.sin(angle + math.pi )
                e.x, e.y = x,y
                enemies.append(e)

            if counter%1200 == 0:
                # make this harder
                if enemy_speed < 15:
                    enemy_speed += 1

            for e in enemies + pills:
                e.move()

            for e in enemies:
                e.new_iter()

            # Check collisions
            for e in enemies:
                if e.dying: continue 

                for p in pills:
                    if p.dead: continue
                    if e.check_collision(p):
                        score.score+=1
                        e.dying = True
                        p.dead = True
                        break
                for e2 in enemies:
                    if e2.dying: continue
                    elif e == e2: continue
                    elif e.check_collision_radial(e2):
                        # Connect to this
                        e.connect(e2)

                if e.check_collision_radial(player):
                    # Player Dying
                    sound.gurglelong()
                    e.dying = True
                    if score.lives > 0:
                        score.lives-=1
                    else:
                        player_dead = True

        # Remove the dead ones
        for i in range(len(enemies)):
            try:
                if enemies[i].dead: 
                    enemies.pop(i)
                    sound.gurgle()
            except: pass

        for i in range(len(pills)):
            try:
                if pills[i].dead: pills.pop(i)
            except: pass


        surface.fill((165,255,165))

        for spr in pills:
            spr.render(surface)

        player.render(surface)
        
        for spr in enemies:
            spr.render(surface)

        score.render(surface)




        for event in pygame.event.get():
            if event.type == QUIT:
                exit_game = True
            elif not player_dead:
                if event.type == KEYDOWN:
                    if event.key == K_ESCAPE:
                        exit_game = True
                    elif event.key == K_SPACE:
                        if t - last_shot > 0.4:
                            last_shot = t
                            p = game.pill.Pill(screen)
                            sound.shot()    
                            p.y = player.y + player.height/2 - p.height/2
                            p.x = player.x + player.width/2 - p.width/2
                            speed = 7
                            p.Vy = math.cos(player.rotation) * speed
                            p.Vx = math.sin(player.rotation) * speed
                            pills.append(p)
                    elif event.key == K_LEFT:
                        rotating = 0.2
                    elif event.key == K_RIGHT:
                        rotating = -0.2
                elif event.type == KEYUP:
                    if event.key == K_LEFT:
                        if rotating == 0.2:
                            rotating = 0
                    if event.key == K_RIGHT:
                        if rotating == -0.2:
                            rotating = 0

        if rotating != 0 and not player_dead:
            player.rotate(angle=rotating)

        counter +=1



if __name__ == "__main__":
    main()
